unit GdiplusSub;

{$IMPORTEDDATA OFF}
{$MINENUMSIZE 4}

interface

uses Windows, ActiveX;

type
  TGpStatus = (
    gpstOk,
    gpstGenericError,
    gpstInvalidParameter,
    gpstOutOfMemory,
    gpstObjectBusy,
    gpstInsufficientBuffer,
    gpstNotImplemented,
    gpstWin32Error,
    gpstWrongState,
    gpstAborted,
    gpstFileNotFound,
    gpstValueOverflow,
    gpstAccessDenied,
    gpstUnknownImageFormat,
    gpstFontFamilyNotFound,
    gpstFontStyleNotFound,
    gpstNotTrueTypeFont,
    gpstUnsupportedGdiplusVersion,
    gpstGdiplusNotInitialized,
    gpstPropertyNotFound,
    gpstPropertyNotSupported
  );

  DebugEventLevel = (DebugEventLevelFatal, DebugEventLevelWarning);

  DebugEventProc = procedure(Level: DebugEventLevel; Message: PChar); stdcall;
  NotificationHookProc = function(out Token: ULONG): TGpStatus; stdcall;
  NotificationUnhookProc = procedure(Token: ULONG); stdcall;

  TGdiplusStartupInput = record
    GdiplusVersion: Cardinal; // UINT32
    DebugEventCallback: DebugEventProc;
    SuppressBackgroundThread: BOOL;
    SuppressExternalCodecs : BOOL;
  end;

  TGdiplusStartupOutput = record
    NotificationHook: NotificationHookProc;
    NotificationUnhook: NotificationUnhookProc;
  end;
  PGdiplusStartupOutput = ^TGdiPlusStartupOutput;

  TGpRect = record
    X: Integer;
    Y: Integer;
    Width: Integer;
    Height: Integer;
  end;

  TBitmapData = record
    Width: UINT;
    Height: UINT;
    Stride: Integer;
    PixelFormat: Integer;
    Scan0: Pointer;
    Reserved: UINT;
  end;

  PPropertyItem = ^TPropertyItem;
  TPropertyItem = record
    Id: PROPID;
    Length: ULONG;
    Type_: Word;
    Value: Pointer;
  end;

const
  ImageLockModeRead = 1;
  ImageLockModeWrite = 2;
  ImageLockModeUserInputBuf = 4;

  PixelFormatIndexed = $00010000;
  PixelFormatGDI = $00020000;
  PixelFormat8bppIndexed = (3 or (8 shl 8) or PixelFormatIndexed or PixelFormatGDI);
  PixelFormat24bppRGB = (8 or (24 shl 8) or PixelFormatGDI);

  ImageFormatGIF: TGUID = '{B96B3CB0-0728-11D3-9D7B-0000F81EF32E}';
  ImageFormatPNG: TGUID = '{B96B3CAF-0728-11D3-9D7B-0000F81EF32E}';

  // Encoder CLSID constants do not exist in the original header.
  // Should use GetImageEncoders.
  CLSID_EncoderJPEG: TGUID = '{557CF401-1A04-11D3-9A73-0000F81EF32E}';
  CLSID_EncoderPNG: TGUID = '{557CF406-1A04-11D3-9A73-0000F81EF32E}';

var
  GdiplusStartup: function (out Token: ULONG;
      const Input: TGdiplusStartupInput; Output: PGdiplusStartupOutput):
      TGpStatus; stdcall;
  GdiplusShutdown: procedure (Token: ULONG); stdcall;

  GdipDisposeImage: function (Image: Pointer): TGpStatus; stdcall;
  GdipGetImageWidth:function (Image: Pointer; var Width: UINT):
      TGpStatus; stdcall;
  GdipGetImageHeight: function (Image: Pointer; var Height: UINT):
      TGpStatus; stdcall;
  GdipGetImageRawFormat: function (Image: Pointer; var Format: TGUID):
      TGpStatus; stdcall;
  GdipGetImagePixelFormat: function (Image: Pointer; var Format: Integer):
      TGpStatus; stdcall;
  GdipGetPropertyCount: function (Image: Pointer; var NumOfProperty: UINT):
      TGpStatus; stdcall;
  GdipGetPropertyIdList: function (Image: Pointer; NumOfProperty: UINT;
      var list: PROPID): TGpStatus; stdcall;
  GdipGetPropertyItemSize: function (Image: Pointer; PropId: PROPID;
      var Size: UINT): TGpStatus; stdcall;
  GdipGetPropertyItem: function (Image: Pointer; PropId: PROPID;
      PropSize: UINT; var Buffer: TPropertyItem): TGpStatus; stdcall;

  GdipCreateBitmapFromStream: function (Stream: IStream; var Bitmap: Pointer):
      TGpStatus; stdcall;
  GdipCreateBitmapFromStreamICM: function (Stream: IStream; var Bitmap: Pointer):
      TGpStatus; stdcall;

  GdipCreateBitmapFromFile: function (const Filename: PWChar;
      var Bitmap: Pointer): TGpStatus; stdcall;
  GdipCreateBitmapFromGdiDib: function (const GdiBitmapInfo: PBitmapInfo;
      GdiBitmapData: Pointer; var Bitmap: Pointer): TGpStatus; stdcall;

  GdipBitmapLockBits: function (Bitmap: Pointer; const Rect: TGpRect;
      Flags: UINT; Format: Integer; var LockedBitmapData: TBitmapData):
      TGpStatus; stdcall;
  GdipBitmapUnlockBits: function (Bitmap: Pointer;
      const LockedBitmapData: TBitmapData): TGpStatus; stdcall;

  GdipSaveImageToFile: function (Image: Pointer; const Filename: PWChar;
      const ClsidEncoder: TGUID; const EncoderParameters: Pointer):
      TGpStatus; stdcall;

implementation

uses NDllLoader;

var
  h: HMODULE;
begin
  h := LoadDll('gdiplus.dll');
  if h = 0 then
    Exit;

  GdiplusStartup := GetProcAddress(h, 'GdiplusStartup');
  GdiplusShutdown := GetProcAddress(h, 'GdiplusShutdown');

  GdipDisposeImage := GetProcAddress(h, 'GdipDisposeImage');
  GdipGetImageWidth := GetProcAddress(h, 'GdipGetImageWidth');
  GdipGetImageHeight := GetProcAddress(h, 'GdipGetImageHeight');
  GdipGetImageRawFormat := GetProcAddress(h, 'GdipGetImageRawFormat');
  GdipGetImagePixelFormat := GetProcAddress(h, 'GdipGetImagePixelFormat');
  GdipGetPropertyCount := GetProcAddress(h, 'GdipGetPropertyCount');
  GdipGetPropertyIdList := GetProcAddress(h, 'GdipGetPropertyIdList');
  GdipGetPropertyItemSize := GetProcAddress(h, 'GdipGetPropertyItemSize');
  GdipGetPropertyItem := GetProcAddress(h, 'GdipGetPropertyItem');

  GdipCreateBitmapFromStream := GetProcAddress(h, 'GdipCreateBitmapFromStream');
  GdipCreateBitmapFromStreamICM := GetProcAddress(h, 'GdipCreateBitmapFromStreamICM');
  GdipCreateBitmapFromFile := GetProcAddress(h, 'GdipCreateBitmapFromFile');
  GdipCreateBitmapFromGdiDib := GetProcAddress(h, 'GdipCreateBitmapFromGdiDib');

  GdipBitmapLockBits := GetProcAddress(h, 'GdipBitmapLockBits');
  GdipBitmapUnlockBits := GetProcAddress(h, 'GdipBitmapUnlockBits');

  GdipSaveImageToFile := GetProcAddress(h, 'GdipSaveImageToFile');
end.
