program NkV;

{$IMPORTEDDATA OFF}

uses NTypes, NLib, NSys, NConf, NForm, NCommand, NResource, MainForm, FileList;

{$R *.RES}

var
  i: Integer;
  keymap: TConf;
  list: TFileList;
  stdin: Boolean;
  path: WideString;
begin
  App.Name := 'NkV';

  ProfDir := GetEnv('NKTOOLSPROF');
  if ProfDir = '' then
    ProfDir := DirName(ProgPath)
  else
    ProfDir := JoinPath([ProfDir, 'nkv']);

  Config := TConf.Create;
  Config.Read(ResourceText('CONFIG'));
  Config.LoadFromFile(JoinPath([ProfDir, 'position']));
  Config.LoadFromFile(JoinPath([ProfDir, 'config']));

  if Config.GetBool('remote') and (Length(Argv) > 1) then
  begin
    for i := 1 to High(Argv) do
      Argv[i] := AbsPath(Argv[i]);
    try
      SendToRemoteForm(TNkVMainForm, JoinCommandLine(SliceArrayW(Argv, 1)));
      Exit;
    except
      on E: EOSError do ;
    end;
  end;

  keymap := TConf.Create;
  keymap.Read(ResourceText('KEYMAP'));
  keymap.LoadFromFile(JoinPath([ProfDir, 'keymap']));

  App.MainForm := TNkVMainForm.Create;
  with App.MainForm as TNkVMainForm do
  begin
    Command.BindKeys(keymap);
    keymap.Free;
    App.Accel := Command.AccelHandle;

    list := TFileList.Create;
    for i := 1 to High(Argv) do
    begin
      path := AbsPath(Argv[i]);
      if IsDir(path) then
      begin
        DirMode := High(Argv) = 1;
        try
          list.AddDir(path);
        except
          on E: EOSError do Error(E.Path, E.Msg);
        end;
      end else
        list.Add(path);
    end;
    list.Sort;
    stdin := list.Count = 0;
    if stdin then
      list.Add('');
    LoadFirstFile(list);
    list.Free;

    if not stdin then
      if FileList.Count = 0 then
      begin
        ExitCode := 1;
        Exit;
      end
      else if FileList.Count = 1 then
      begin
        DirMode := True;
        FileListUpdate := True;
      end;
    Show;
  end;
  ExitCode := App.Run;

  App.MainForm.Free;
  Config.Free;
end.
